/* Copyright (C) 2005-2019 Shugo Maeda <shugo@ruby-lang.org> and Charlie Savage <cfis@savagexi.com>
   Please see the LICENSE file for copyright and distribution information */

#ifndef __BF_PROFILE_H__
#define __BF_PROFILE_H__

#include "bf_measurement.h"
#include "bf_thread.h"
#include "blackfire.h"

#define THRESHOLD_MAX_LEN 34
#define LLONG_MAX_LEN     26 * 4
#define START_MAX_LEN     THRESHOLD_MAX_LEN + LLONG_MAX_LEN


extern VALUE cProfile;

typedef struct prof_timeline_item_t
{
    long long wt;
    long long pr;
    long long al;
    long long mm;
    char *fqmn;
    struct prof_timeline_item_t *caller;
} prof_timeline_item_t;

typedef struct prof_profile_t
{
    /* header data */
    VALUE bf_version;
    VALUE context;
    VALUE title;
    VALUE probed_features;
    VALUE ruby_description;
    VALUE ruby_release_date;
    bool timeline_defined;
    VALUE timeline_rules;
    int timeline_rules_count;
    bool header_was_built;

    /* internal data */
    prof_measurer_t *measurer;
    struct prof_timeline_item_t *current_method;
    st_table *threads_tbl;
    st_table *exclude_threads_tbl;
    st_table *include_threads_tbl;
    st_table *exclude_methods_tbl;
    thread_data_t *last_thread_data;
    struct four_doubles_t measurement_at_pause_resume;
    bool allow_exceptions;
    bool merge_fibers;
    int current_threshold;
    bool first_threshold;
    long long threshold_initial_time;
    long long timespan_threshold;
    rb_event_flag_t last_event;

    /* outputs  */
    VALUE running;
    VALUE paused;
    VALUE tracepoints;
    VALUE timelines;
    VALUE headers;
    VALUE measurements;
} prof_profile_t;

void rp_init_profile(void);
prof_profile_t *prof_get_profile(VALUE self);

#endif //__BF_PROFILE_H__
