# Load the C-based binding.
begin
  RUBY_VERSION =~ /(\d+\.\d+\.\d+)/
  require "#{Regexp.last_match(1)}/blackfire.so"
rescue LoadError
  require "blackfire.so"
end

require "blackfire/bffnargs"
require "blackfire/bffnargs_rule"
require "blackfire/bfyaml"
require "blackfire/call_tree"
require "blackfire/config"
require "blackfire/logger"
require "blackfire/measurement"
require "blackfire/method_info"
require "blackfire/printer_helper"
require "blackfire/profile"
require "blackfire/protocol"
require "blackfire/query"
require "blackfire/query_trace"
require "blackfire/query_details"
require "blackfire/rack"
require "blackfire/thread"
require "blackfire/version"
require "patches/sql_patches" if defined?(::Rails) && defined?(::Rails::VERSION) && ::Rails::VERSION::MAJOR.to_i >= 4

# rubocop:disable Style/RedundantBegin
# rubocop:disable Style/RescueStandardError
module Blackfire
  class << self
    def inside_rails?
      defined?(::Rails) && defined?(::Rails::VERSION) && ::Rails::VERSION::MAJOR.to_i >= 4
    end
  end

  autoload :CallTreeVisitor, "blackfire/call_tree_visitor"
  autoload :AbstractPrinter, "blackfire/printers/abstract_printer"
  autoload :CallInfoPrinter, "blackfire/printers/call_info_printer"
  autoload :FlatPrinter, "blackfire/printers/flat_printer"
  autoload :MultiPrinter, "blackfire/printers/multi_printer"
  autoload :TimelineItem, "blackfire/timeline_item"
  autoload :InstallGenerator, "generators/blackfire/install_generator" if inside_rails?

  Config.initialize
  Logger.initialize
end
# rubocop:enable Style/RedundantBegin
# rubocop:enable Style/RescueStandardError
