module Blackfire
  # This is the base class for all Printers. It is never used directly.
  class AbstractPrinter
    # Create a new printer.
    #
    # result should be the output generated from a profiling run
    def initialize(result)
      @result = result
      @output = nil
    end

    # Returns how profile data should be sorted
    def sort_method
      @options[:sort_method]
    end

    # Prints a report to the provided output.
    #
    # output - Any IO object, including STDOUT or a file.
    # The default value is STDOUT.
    #
    # options - Hash of print options. Note that each printer can
    # define its own set of options.
    #
    #   :min_percent - Number 0 to 100 that specifes the minimum
    #                  %self (the methods self time divided by the
    #                  overall total time) that a method must take
    #                  for it to be printed out in the report.
    #                  Default value is 0.
    #
    #   :sort_method - Specifies method used for sorting method infos.
    #                  Available values are :total_time, :self_time,
    #                  :wait_time, :children_time
    #                  Default value is :total_time
    # rubocop:disable Style/OptionHash
    def print(output = STDOUT, options = {})
      @output = output
      print_threads
    end
    # rubocop:enable Style/OptionHash

    def print_threads
      @result.threads.each do |thread|
        print_thread(thread)
      end
    end

    def print_thread(thread)
      print_methods(thread)
    end
  end
end
