module Blackfire
  class FlatPrinter < AbstractPrinter
    include PrinterHelper
    attr_accessor :query_current_order, :time_to_remove

    def increment_query_current_order(call_tree)
      original_method = call_tree.target.full_name.split("#")[1]
      return if QueryTrace.exec_methods.nil?

      @query_current_order += 1 if QueryTrace.exec_methods.include? original_method
    end

    # rubocop:disable Metrics/CyclomaticComplexity
    def print_methods(thread)
      visitor = CallTreeVisitor.new(thread.call_tree)
      first = true
      metrics = ""
      @query_current_order = -1
      @time_to_remove = nil
      root_method = ""

      visitor.visit do |call_tree, event|
        case event
        when :enter
          if first
            root_method = call_tree.target.full_name
            @output << "Graph-Root-Id: #{root_method}\n"
            first = false
          end

          increment_query_current_order(call_tree)
          metrics << format(
            "%s//%s %s %s %s %s\n",
            build_target_name(call_tree, @query_current_order),
            call_tree.called.to_s,
            (call_tree.wall_time_total_time * MULTIPLIER_FOR_TIME).to_i.to_s,
            (call_tree.process_total_time * MULTIPLIER_FOR_TIME).to_i.to_s,
            call_tree.allocation_total_time.to_i.to_s,
            (call_tree.memory_total_time * MULTIPLIER_FOR_MEMORY).to_i.to_s
          )
        end
      end

      @output << metrics
    end
    # rubocop:enable Metrics/CyclomaticComplexity
  end
end
