module Blackfire
  module QueryTrace
    class << self
      attr_accessor :queries, :order, :exec_methods, :prepared_queries

      def initialize
        cleanup
      end

      def record_prepared_query(id, sql)
        @prepared_queries[id] = sql
      end

      # more details at https://edgeguides.rubyonrails.org/active_record_querying.html
      def record_sql(sql, params)
        query = if @prepared_queries.keys.include? sql
                  @prepared_queries[sql]
                else
                  sql
                end
        @exec_methods << "exec_query"
        qd = QueryDetails.new(build_query(query, params), @order, "exec_query")
        @queries << qd
        @order += 1
      end

      def cleanup
        @queries = []
        @order = 0
        @exec_methods = []
        @prepared_queries = {} if @prepared_queries.nil?
      end

      def report_sql(method, order)
        @queries.each do |query|
          if query.method == method && query.order == order
            sql_query = query.sql
            sql_query = sql_query.gsub("%", "%25")
            sql_query = sql_query.gsub("+", "%2B")
            sql_query = sql_query.gsub("=", "%3D")
            sql_query = sql_query.gsub("@", "%40")
            sql_query = sql_query.gsub("&", "%26")
            sql_query = sql_query.gsub("\r", "%0D")
            sql_query = sql_query.gsub("\n", "%0A")
            return sql_query
          end
        end
        nil
      end

      private

      def build_query(sql, params)
        return sql if params.nil?
        return sql unless sql.count("?") == params.length
        return sql if params.length.zero?

        ret = ""
        data = sql.split("?")

        1.upto(params.length) do |i|
          ret += data[i - 1].to_s
          ret += params[i - 1].value.to_s
        end

        ret
      end
    end
  end
end
