# frozen_string_literal: true

module Rack
  class BlackfireRack
    module ActiveRecordInstrumentation
      def self.included(instrumented_class)
        instrumented_class.class_eval do
          unless instrumented_class.method_defined?(:log_without_blackfire)
            alias_method :log_without_blackfire, :log
            alias_method :log, :log_with_blackfire
            protected :log
          end
        end
      end

      def log_with_blackfire(*args, &block)
        return log_without_blackfire(*args, &block) unless SqlPatches.should_measure?

        sql, binds = args
        rval = log_without_blackfire(*args, &block)
        Blackfire::QueryTrace.record_sql(sql, binds)
        rval
      end
    end
  end

  def self.insert_instrumentation
    ActiveRecord::ConnectionAdapters::AbstractAdapter.module_eval do
      include BlackfireRack::ActiveRecordInstrumentation
    end
  end

  insert_instrumentation if defined?(::Rails)
end
