# frozen_string_literal: true

# Based on https://github.com/MiniProfiler/rack-mini-profiler/tree/master/lib/patches
module Rack
  class BlackfireRack
    class SqlPatches
      def self.correct_version?(required_version, klass)
        Gem::Dependency.new("", required_version).match?("", klass::VERSION)
      rescue NameError
        false
      end

      def self.record_sql(statement, parameters = nil)
        result = yield
        record = ::Blackfire.record_sql(statement, parameters)
        [result, record]
      end

      def self.should_measure?
        Blackfire.check_running? && Blackfire::Query.flag_pdo == "1"
      end

      # rubocop:disable Metrics/CyclomaticComplexity
      # rubocop:disable Metrics/PerceivedComplexity
      def self.sql_patches
        patches = []

        patches << "mysql2" if defined?(Mysql2::Client) && Mysql2::Client.class == Class
        patches << "pg" if defined?(PG::Result) && PG::Result.class == Class

        return patches unless patches.empty?

        patches << "sequel" if defined?(Sequel::Database) && Sequel::Database.class == Class
        patches << "activerecord" if defined?(ActiveRecord) && ActiveRecord.class == Module
        patches
      end
      # rubocop:enable Metrics/CyclomaticComplexity
      # rubocop:enable Metrics/PerceivedComplexity

      def self.all_patch_files
        env_var = ENV["BLACKFIRE_RACK_PATCH"]
        return [] if env_var == "false"

        env_var ? env_var.split(",").map(&:strip) : sql_patches
      end

      def self.patch(patch_files = all_patch_files)
        patch_files.each do |patch_file|
          require "patches/db/#{patch_file}"
        end
      end
    end
  end
end
